@@ -52,6 +52,7 @@
 #include "sj-main.h"
 #include "sj-prefs.h"
 #include "sj-play.h"
+#include "gedit-message-area.h"
 #include "bacon-volume.h"
 
 gboolean on_delete_event (GtkWidget *widget, GdkEvent *event, gpointer user_data);
@@ -71,6 +72,7 @@ SjExtractor *extractor;
 GConfClient *gconf_client;
 
 GtkWidget *main_window;
+static GtkWidget *message_area_vbox;
 static GtkWidget *title_entry, *artist_entry, *duration_label, *genre_entry;
 static GtkWidget *track_listview, *extract_button, *play_button;
 static GtkWidget *status_bar;
@@ -80,6 +82,8 @@ GtkListStore *track_store;
 static BaconMessageConnection *connection;
 GtkCellRenderer *toggle_renderer, *title_renderer, *artist_renderer;
 
+GtkWidget *current_message_area;
+
 const char *path_pattern, *file_pattern;
 char *base_uri;
 NautilusBurnDrive *drive = NULL;
@@ -345,6 +349,126 @@ static void number_cell_icon_data_cb (GtkTreeViewColumn *tree_column,
   }
 }
 
+/* Taken from gedit */
+static void
+set_message_area_text_and_icon (GeditMessageArea *message_area,
+				const gchar   *icon_stock_id,
+				const gchar   *primary_text,
+				const gchar   *secondary_text)
+{
+  GtkWidget *hbox_content;
+  GtkWidget *image;
+  GtkWidget *vbox;
+  gchar *primary_markup;
+  gchar *secondary_markup;
+  GtkWidget *primary_label;
+  GtkWidget *secondary_label;
+
+  hbox_content = gtk_hbox_new (FALSE, 8);
+  gtk_widget_show (hbox_content);
+
+  image = gtk_image_new_from_stock (icon_stock_id, GTK_ICON_SIZE_DIALOG);
+  gtk_widget_show (image);
+  gtk_box_pack_start (GTK_BOX (hbox_content), image, FALSE, FALSE, 0);
+  gtk_misc_set_alignment (GTK_MISC (image), 0.5, 0);
+
+  vbox = gtk_vbox_new (FALSE, 6);
+  gtk_widget_show (vbox);
+  gtk_box_pack_start (GTK_BOX (hbox_content), vbox, TRUE, TRUE, 0);
+
+  primary_markup = g_strdup_printf ("<b>%s</b>", primary_text);
+  primary_label = gtk_label_new (primary_markup);
+  g_free (primary_markup);
+  gtk_widget_show (primary_label);
+  gtk_box_pack_start (GTK_BOX (vbox), primary_label, TRUE, TRUE, 0);
+  gtk_label_set_use_markup (GTK_LABEL (primary_label), TRUE);
+  gtk_label_set_line_wrap (GTK_LABEL (primary_label), TRUE);
+  gtk_misc_set_alignment (GTK_MISC (primary_label), 0, 0.5);
+
+  if (secondary_text != NULL) {
+    secondary_markup = g_strdup_printf ("<small>%s</small>",
+					secondary_text);
+    secondary_label = gtk_label_new (secondary_markup);
+    g_free (secondary_markup);
+    gtk_widget_show (secondary_label);
+    gtk_box_pack_start (GTK_BOX (vbox), secondary_label, TRUE, TRUE, 0);
+    gtk_label_set_use_markup (GTK_LABEL (secondary_label), TRUE);
+    gtk_label_set_line_wrap (GTK_LABEL (secondary_label), TRUE);
+    gtk_misc_set_alignment (GTK_MISC (secondary_label), 0, 0.5);
+  }
+
+  gedit_message_area_set_contents (GEDIT_MESSAGE_AREA (message_area),
+				   hbox_content);
+}
+
+/* Taken from gedit */
+static void
+set_message_area (GtkWidget *container,
+		  GtkWidget *message_area)
+{
+  if (current_message_area == message_area)
+      return;
+
+  if (current_message_area != NULL)
+    gtk_widget_destroy (current_message_area);
+
+  current_message_area = message_area;
+
+  if (message_area == NULL)
+    return;
+
+  gtk_box_pack_start (GTK_BOX (container),
+		      current_message_area,
+		      FALSE,
+		      FALSE,
+		      0);
+
+  g_object_add_weak_pointer (G_OBJECT (current_message_area),
+			     (gpointer *)&current_message_area);
+}
+
+static GtkWidget* musicbrainz_submit_message_area_new (char *title,
+						       char *artist)
+{
+  GtkWidget *message_area;
+  char *primary_text;
+
+  g_return_val_if_fail (title != NULL, NULL);
+  g_return_val_if_fail (artist != NULL, NULL);
+
+  message_area = gedit_message_area_new ();
+
+  gedit_message_area_add_button (GEDIT_MESSAGE_AREA (message_area),
+			      _("_Submit Album"),
+			      GTK_RESPONSE_OK);
+  gedit_message_area_add_button (GEDIT_MESSAGE_AREA (message_area),
+			      GTK_STOCK_CANCEL,
+			      GTK_RESPONSE_CANCEL);
+
+  /* Translators: title, artist */
+  primary_text = g_strdup_printf (_("Could not find %s by %s on MusicBrainz."), title, artist);
+
+  set_message_area_text_and_icon (GEDIT_MESSAGE_AREA (message_area),
+				  "gtk-dialog-info",
+				  primary_text,
+				  _("You can improve the MusicBrainz database by adding this album."));
+
+  g_free (primary_text);
+
+  return message_area;
+}
+
+static void musicbrainz_submit_message_area_response (GeditMessageArea *message_area,
+						      gint           response_id,
+						      gpointer       user_data)
+{
+  if (response_id == GTK_RESPONSE_OK) {
+    on_submit_activate (NULL, NULL);
+  }
+
+  set_message_area (message_area_vbox, NULL);
+}
+
 /**
  * Utility function to update the UI for a given Album
  */
@@ -375,6 +499,8 @@ static void update_ui_for_album (AlbumDetails *album)
     gtk_widget_set_sensitive (deselect_all_menuitem, FALSE);
     gtk_widget_set_sensitive (prev_menuitem, FALSE);
     gtk_widget_set_sensitive (next_menuitem, FALSE);
+
+    set_message_area (message_area_vbox, NULL);
   } else {
     gtk_list_store_clear (track_store);
 
@@ -425,12 +551,31 @@ static void update_ui_for_album (AlbumDetails *album)
     } else {
       gtk_label_set_text (GTK_LABEL (duration_label), _("(unknown)"));
     }
+
+    /* If album details don't come from MusicBrainz ask user to add them */
+    if (album->metadata_source != SOURCE_MUSICBRAINZ) {
+      GtkWidget *message_area;
+
+      message_area = musicbrainz_submit_message_area_new (album->title,
+							  album->artist);
+
+      set_message_area (message_area_vbox, message_area);
+
+      g_signal_connect (message_area,
+			"response",
+			G_CALLBACK (musicbrainz_submit_message_area_response),
+			NULL);
+
+      gedit_message_area_set_default_response (GEDIT_MESSAGE_AREA (message_area),
+					    GTK_RESPONSE_CANCEL);
+
+      gtk_widget_show (message_area);
+    }
   }
 }
 
 /**
- * Called by the Multiple Album dialog when the user hits return in
- * the list view
+ * Callback that gets fired when a user double clicks on a row
  */
 static void album_row_activated (GtkTreeView *treeview,
                                  GtkTreePath *arg1,
@@ -441,6 +586,22 @@ static void album_row_activated (GtkTreeView *treeview,
   gtk_dialog_response (dialog, GTK_RESPONSE_OK);
 }
 
+/**
+ * Callback that gets fired when an the selection changes. We use this to
+ * change the sensitivity of the continue button
+ */
+static void selected_album_changed (GtkTreeSelection *selection, 
+                                    gpointer *user_data)
+{
+  GtkWidget *ok_button = GTK_WIDGET (user_data);
+
+  if (gtk_tree_selection_get_selected (selection, NULL, NULL)) {
+    gtk_widget_set_sensitive (ok_button, TRUE);
+  } else {
+    gtk_widget_set_sensitive (ok_button, FALSE);
+  }
+}
+
 /**
  * Utility function for when there are more than one albums available
  */
@@ -452,6 +613,7 @@ AlbumDetails* multiple_album_dialog(GList *albums)
   AlbumDetails *album;
   GtkTreeIter iter;
   int response;
+  GtkWidget *ok_button = NULL;
 
   if (dialog == NULL) {
     GtkTreeViewColumn *column;
@@ -461,7 +623,8 @@ AlbumDetails* multiple_album_dialog(GList *albums)
     g_assert (dialog != NULL);
     gtk_window_set_transient_for (GTK_WINDOW (dialog), GTK_WINDOW (main_window));
     albums_listview = glade_xml_get_widget (glade, "albums_listview");
-    /* TODO: A little hacky */
+    ok_button = glade_xml_get_widget (glade, "ok_button");
+
     g_signal_connect (albums_listview, "row-activated", G_CALLBACK (album_row_activated), dialog);
 
     albums_store = gtk_list_store_new (3, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_POINTER);
@@ -479,6 +642,8 @@ AlbumDetails* multiple_album_dialog(GList *albums)
     gtk_tree_view_set_model (GTK_TREE_VIEW (albums_listview), GTK_TREE_MODEL (albums_store));
     selection = gtk_tree_view_get_selection(GTK_TREE_VIEW (albums_listview));
     gtk_tree_selection_set_mode(selection, GTK_SELECTION_BROWSE);
+    gtk_widget_set_sensitive (ok_button, FALSE);
+    g_signal_connect (selection, "changed", (GCallback)selected_album_changed, ok_button);
   }
 
   gtk_list_store_clear (albums_store);
@@ -492,11 +657,13 @@ AlbumDetails* multiple_album_dialog(GList *albums)
                         2, album,
                         -1);
   }
-  /*
-   * TODO: focus is a little broken here. The first row should be
-   * selected, so just hitting return works.
-   */
-  gtk_widget_grab_focus (albums_listview);
+
+  /* Select the first album */
+  if (gtk_tree_model_get_iter_first (GTK_TREE_MODEL (albums_store), &iter))
+  {
+    gtk_tree_selection_select_iter (selection, &iter);
+  }
+
   gtk_widget_show_all (dialog);
   response = gtk_dialog_run (GTK_DIALOG (dialog));
   gtk_widget_hide (dialog);
@@ -504,9 +671,13 @@ AlbumDetails* multiple_album_dialog(GList *albums)
   if (response == GTK_RESPONSE_DELETE_EVENT) {
     return NULL;
   }
-  gtk_tree_selection_get_selected (selection, NULL, &iter);
-  gtk_tree_model_get (GTK_TREE_MODEL (albums_store), &iter, 2, &album, -1);
-  return album;
+
+  if (gtk_tree_selection_get_selected (selection, NULL, &iter)) {
+    gtk_tree_model_get (GTK_TREE_MODEL (albums_store), &iter, 2, &album, -1);
+    return album;
+  } else {
+    return NULL;
+  }
 }
 
 /**
@@ -1400,6 +1571,7 @@ int main (int argc, char **argv)
   glade_xml_signal_autoconnect (glade);
 
   main_window = glade_xml_get_widget (glade, "main_window");
+  message_area_vbox = glade_xml_get_widget (glade, "message_area_vbox");
   select_all_menuitem = glade_xml_get_widget (glade, "select_all");
   deselect_all_menuitem = glade_xml_get_widget (glade, "deselect_all");
   submit_menuitem = glade_xml_get_widget (glade, "submit");
